% Gill model written as relaxation towards prescribed height

function D = IITCZ_SWM(R)
% Written by Ori Adam Jun.2017
% Contact: ori.adam@mail.huji.ac.il
D.Descriptor = 'IITCZ_SWM'     ; % Used to identify file version, for compatibility with later versions
%% 2D Linearized SWM on the equatorial Beta plane
% The Algorithm uses Leap-frog to integrate the equations with an Asselin filter
% The integration is initialized with a forward euler routine for the first step
% The nondimensional equations are:
% 1) d/dt u =  -y*v - dh/dx - epsilon*u
% 2) d/dt v =  +y*u - dh/dy - epsilon*v
% 3) d/dt h =  - (du/dx + dv/dy)  - Q
% where:   - x is 'longitude'
%          - y is 'latitude'
%          - v = 'meridional' component of velocity
%          - u = 'zonal' conponent of velocity
%          - h = geopotential height of free surface
%          - Q = Source
% Staggered grid is used. h and u are staggered from v along y. v = 0 at polar boundaries.

D = Defaults(D)  ; % Set Default Parameters
D.MainFig = figure('name','Matsuno-Gill SWM','position',D.pos) ;
set(D.MainFig,'Color',D.FigBackgroudColor) ;
D = Dependent_Consts(D)                  ;
if nargin
    try
        RFieldNames = fieldnames(R) ;
        for j = 1:length(RFieldNames)
            D.(RFieldNames{j}) = R.(RFieldNames{j}) ;
        end
    catch
        disp('Error: function input ''R'' is a non-structure array!')
    end
end

% Initial display = Initial Conditions
D = InitialConditions(D)       ; % Set Initial Conditions
D = CreateGUI(D) ; % Crate GUI, buttons, etc..
frame_plot(D) ; drawnow        ; % Update screen
try
    set(D.MainFig,'name',['Matsuno-Gill SWM -- Case = ',regexprep(D.ForcingString,'_','\_')])
end
D = RunSimulation(D)                     ; % Begin Time Integration

% Internal Functions
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Progress plot
function frame_plot(D)
dx = ceil(D.Nx/25) ;
dy = ceil(D.Ny/10) ;
hf = -(D.Qx + D.Qy)/D.epsilon ;
YL = 3 ;
subplot (2,14,[2 7])
switch  D.DisplayIdx
    case {1,2}
        Cfactor = max(abs(D.u(:)))*1.01 ;
        levels = linspace(-Cfactor,Cfactor,21) ;
        [C,h] = contourf(D.Xh,D.Yh,D.u,levels) ;  colorbar ;  hold on
        R= -D.vorh./D.Yh ; R(abs(D.Yh)<.1) = 0 ;
        Rfactor = max(abs(R(:)))*1.01 ;
        contour(D.Xh,D.Yh,R,[0:.2:1]*Rfactor,'color',[1 1 1]*.1,'linewidth',1,'linestyle','-') ;
        contour(D.Xh,D.Yh,R,[0:.2:1]*-Rfactor,'color',[1 1 1]*.1,'linewidth',1,'linestyle','--') ;
        caxis([-1 1]*Cfactor) ;
        %     if D.u_ContourLevelStep
        %     set(h,'levelstep',D.u_ContourLevelStep) ;
        %     end
        xlabel('\bfx','fontsize',16) ;
        ylabel('\bfy','fontsize',16) ;
        title('           \bf\itu \rm(color) and \itRo \rm(contour)                 ','fontsize',16) ;
        if D.LineWidth > 0
            try  set(h,'linewidth',D.LineWidth) ; end
        else
            try  set(h,'linestyle','none') ; end
        end
        set(gca,'fontsize',12,'xtick',0:5:25,'xticklabels',{'-10','-5','0','+5','+10','+15'},'box','off')
        xlim([0 D.Lx])
        ylim([-1 1]*YL)
end
subplot (2,14,[9 14])
switch D.DisplayIdx
    case {1,2}
        
        Cfactor = max(abs(D.h(:)))*1.01 ;
        levels = linspace(-Cfactor,Cfactor,21) ;
        [C,h] = contourf(D.Xh,D.Yh,D.h,levels) ;  colorbar ; hold on ;
        [Cp,hp] = contour(D.Xh,D.Yh,hf,[0.01:0.01:1]*1,'color',[1 1 1]*.2,'linewidth',1,'linestyle','-') ;
        [Cm,hm] = contour(D.Xh,D.Yh,hf,-[0.01:0.01:1]*1,'color',[1 1 1]*.2,'linewidth',1,'linestyle','--') ;
        caxis([-1 1]*Cfactor) ;
        if D.h_ContourLevelStep
            set(h,'levelstep',D.h_ContourLevelStep) ;
        end
        xlabel('\bfx','fontsize',16) ;
        ylabel('\bfy','fontsize',16) ;
        title('           \bf\ith \rm(color) and \it\bfh_f \rm(contour)                       ','fontsize',16) ;
        if D.LineWidth > 0
            try  set(h,'linewidth',D.LineWidth) ; end
        else
            try  set(h,'linestyle','none') ; end
        end
        set(gca,'fontsize',12,'xtick',0:5:25,'xticklabels',{'-10','-5','0','+5','+10','+15'},'box','off')
        xlim([0 D.Lx])
        ylim([-1 1]*YL)
end
subplot (2,14,[16 21])
switch D.DisplayIdx
    case {1,2}
        Cfactor = max(abs(D.v(:)))*1.01 ;
        Vfactor = max(abs(D.vor(:)))*1.01 ;
        levels = linspace(-Cfactor,Cfactor,21) ;
        [C,h] = contourf(D.X,D.Y,D.v,levels) ;  colorbar ; hold on
        D = CalculateVorDiv(D) ;
        contour(D.X,D.Y,D.vor,[0:.2:1]*Vfactor,'color',[1 1 1]*.1,'linewidth',1,'linestyle','-') ; hold on
        contour(D.X,D.Y,D.vor,[0:.2:1]*-Vfactor,'color',[1 1 1]*.1,'linewidth',1,'linestyle','--') ;
        caxis([-1 1]*Cfactor) ;
        xlabel('\bfx','fontsize',16) ;
        ylabel('\bfy','fontsize',16) ;
        title('           \bf\itv \rm(color)  and \bf\it \xi \rm(contour)                        ','fontsize',16) ;
        if D.LineWidth > 0
            try  set(h,'linewidth',D.LineWidth) ; end
        else
            try  set(h,'linestyle','none') ; end
        end
        set(gca,'fontsize',12,'xtick',0:5:25,'xticklabels',{'-10','-5','0','+5','+10','+15'},'box','off')
        xlim([0 D.Lx])
        ylim([-1 1]*YL)
end
subplot (2,14,[23 28])
switch D.DisplayIdx
    case {1,2}
        %Q = D.Q - D.divh*(1-D.gamma) ;
        %Q = D.Qx ;
        %Q = D.vorQ ;
        Q = - D.divh ;
        %Q = D.Qx ;
        %Q = D.Qo ;
        Cfactor = max(abs(Q(:)))*1.01 ;
        levels = linspace(-Cfactor,Cfactor,21) ;
        [C,h] = contourf(D.Xh,D.Yh,Q,levels) ; colormap(D.ColorMaps.TWBceJulien)  ; colorbar ; hold on
        u   = interp2(D.x',D.yh,D.u,D.x',D.y,'spline')  ;
        q = quiver(D.x(1:dx:end),D.y(1:dy:end),u(1:dx:end,1:dy:end),D.v(1:dx:end,1:dy:end)) ;
        q.Color = [1 1 1]*.1 ;
        caxis([-1 1]*Cfactor) ;
        xlabel('\bfx','fontsize',16) ;
        ylabel('\bfy','fontsize',16) ;
        title('           \bf\it w ~ Q ~ -(\partial_xu + \partial_yv)         ','fontsize',16) ;
        %     if D.epsilon0 == 0
        %     title('           \bf\itQ = (h - h_f)/\tau ~ w                          ','fontsize',16) ;
        %     else
        %     title('           \bf\itQ = (h - h_f)/\tau + \alpha \epsilon \xi/y ~ w                          ','fontsize',16) ;
        %     end
        if D.LineWidth > 0
            try  set(h,'linewidth',D.LineWidth) ; end
        else
            try  set(h,'linestyle','none') ; end
        end
        set(gca,'fontsize',12,'xtick',0:5:25,'xticklabels',{'-10','-5','0','+5','+10','+15'},'box','off')
        xlim([0 D.Lx])
        ylim([-1 1]*YL)
end
% Calculate Vorticity and Divergence
function D = CalculateVorDiv(D)
D.vor(:,2:end-1) = (D.vk(:,3:end)- D.vk(:,1:end-2))/(2*D.dx) ;
D.vor(:,1) = (D.vk(:,2)- D.vk(:,end))/(2*D.dx) ;
D.vor(:,end) = (D.vk(:,1)- D.vk(:,end-1))/(2*D.dx) ;
D.vor(2:end-1,:) = D.vor(2:end-1,:) - (D.uk(2:end,:) - D.uk(1:end-1,:))/D.dy ;
D.vorh = (D.vor(2:end,:) + D.vor(1:end-1,:))/2 ; %

if D.Total_Ekman_Flag % zonal + meridional ocean divergence
    if D.Linear_Ekman_Flag
        %D.Qo(2:D.Ny-2,:) = - D.C./(D.y0 + D.Yh(2:D.Ny-2,:).^2).*((D.uk(3:D.Ny-1,:) - D.uk(1:D.Ny-3,:))/(2*D.dy).*D.Yh(2:D.Ny-2,:) - D.uk(2:D.Ny-2,:)) ;
        D.Qo(2:D.Ny-2,:) =  D.C/(2*D.dy).*(D.Yh(3:D.Ny-1,:).*D.uk(3:D.Ny-1,:)./(D.y0^2 + D.Yh(3:D.Ny-1,:).^2) - D.Yh(1:D.Ny-3,:).*D.uk(1:D.Ny-3,:)./(D.y0^2 + D.Yh(1:D.Ny-3,:).^2)) ;
        D.Qoz(:,2:D.Nx-1) =  -D.C/(2*D.dx)*(D.Yh(:,3:D.Nx).*(D.vk(1:D.Ny-1,3:D.Nx) + D.vk(2:D.Ny,3:D.Nx) )/2./(D.y0^2 + D.Yh(:,3:D.Nx).^2) - D.Yh(:,1:D.Nx-2).*(D.vk(1:D.Ny-1,1:D.Nx-2)+D.vk(2:D.Ny,1:D.Nx-2))/2./(D.y0^2 + D.Yh(:,1:D.Nx-2).^2)) ;
        D.Qoz(:,1)        =  -D.C/(2*D.dx)*(D.Yh(:,2     ).*(D.vk(1:D.Ny-1,2     ) + D.vk(2:D.Ny,2     ))/2./(D.y0^2 + D.Yh(:,2).^2     ) - D.Yh(:,D.Nx    ).*(D.vk(1:D.Ny-1,D.Nx    ) + D.vk(2:D.Ny,D.Nx    ))/2./(D.y0^2 + D.Yh(:,D.Nx    ).^2)) ;
        D.Qoz(:,D.Nx)     =  -D.C/(2*D.dx)*(D.Yh(:,1     ).*(D.vk(1:D.Ny-1,1     ) + D.vk(2:D.Ny,1     ))/2./(D.y0^2 + D.Yh(:,1).^2     ) - D.Yh(:,D.Nx-1  ).*(D.vk(1:D.Ny-1,D.Nx-1  ) + D.vk(2:D.Ny,D.Nx-1  ))/2./(D.y0^2 + D.Yh(:,D.Nx-1  ).^2)) ;
    else
        D.Qo(2:D.Ny-2,:) = - D.C.*sign(D.uk(2:D.Ny-2,:)).*(D.uk(2:D.Ny-2,:).*(D.uk(3:D.Ny-1,:) - D.uk(1:D.Ny-3,:))*2/(2*D.dy).*D.Yh(2:D.Ny-2,:)./(D.y0 + D.Yh(2:D.Ny-2,:).^2)  ...
            - D.uk(2:D.Ny-2,:).^2./(D.y0 + D.Yh(2:D.Ny-2,:).^2)) ;
        % Zonal term in the ocean divergence
        D.Qoz(:,2:D.Nx-1) =  D.Cx*2.*abs(D.vk(1:D.Ny-1,2:D.Nx-1) + D.vk(2:D.Ny,2:D.Nx-1))/2.*((D.vk(1:D.Ny-1,3:D.Nx) - D.vk(1:D.Ny-1,1:D.Nx-2))/2 + (D.vk(2:D.Ny,3:D.Nx) - D.vk(2:D.Ny,1:D.Nx-2))/2)/(2*D.dx).*D.Yh(:,2:D.Nx-1)./(D.y0 + D.Yh(:,2:D.Nx-1).^2) ;
        D.Qoz(:,1)        =  D.Cx*2.*abs(D.vk(1:D.Ny-1,1       ) + D.vk(2:D.Ny,1       ))/2.*((D.vk(1:D.Ny-1,2     ) - D.vk(1:D.Ny-1,D.Nx)    )/2 + (D.vk(2:D.Ny,2     ) - D.vk(2:D.Ny,D.Nx)    )/2)/(2*D.dx).*D.Yh(:,1       )./(D.y0 + D.Yh(:,1       ).^2) ;
        D.Qoz(:,D.Nx)     =  D.Cx*2.*abs(D.vk(1:D.Ny-1,D.Nx    ) + D.vk(2:D.Ny,D.Nx    ))/2.*((D.vk(1:D.Ny-1,1     ) - D.vk(1:D.Ny-1,D.Nx-1)  )/2 + (D.vk(2:D.Ny,1     ) - D.vk(2:D.Ny,D.Nx-1)  )/2)/(2*D.dx).*D.Yh(:,D.Nx    )./(D.y0 + D.Yh(:,D.Nx).^2) ;
    end
    D.Qo = D.Qo + D.Qoz ;
    if D.Ekman_Flag
        D.Qo(D.Qo>0)=0 ;
    end
elseif D.Ekman_Flag
    if D.Linear_Ekman_Flag
        D.Qo(2:D.Ny-2,:) = - D.C*(sign(-D.uk(2:D.Ny-2,:))+1)/2./(D.y0 + D.Yh(2:D.Ny-2,:).^2).*((D.uk(3:D.Ny-1,:) - D.uk(1:D.Ny-3,:))/(2*D.dy).*D.Yh(2:D.Ny-2,:) - D.uk(2:D.Ny-2,:)) ;
    else
        D.Qo(2:D.Ny-2,:) = - (sign(-D.uk(2:D.Ny-2,:))+1)/2.* D.C.*sign(D.uk(2:D.Ny-2,:)).*(D.uk(2:D.Ny-2,:).*(D.uk(3:D.Ny-1,:) - D.uk(1:D.Ny-3,:))*2/(2*D.dy).*D.Yh(2:D.Ny-2,:)./(D.y0 + D.Yh(2:D.Ny-2,:).^2)  ...
            - D.uk(2:D.Ny-2,:).^2./(D.y0 + D.Yh(2:D.Ny-2,:).^2)) ;
    end
else
    if D.Linear_Ekman_Flag
        D.Qo(2:D.Ny-2,:) = - D.C./(D.y0 + D.Yh(2:D.Ny-2,:).^2).*((D.uk(3:D.Ny-1,:) - D.uk(1:D.Ny-3,:))/(2*D.dy).*D.Yh(2:D.Ny-2,:) - D.uk(2:D.Ny-2,:)) ;
    else
        D.Qo(2:D.Ny-2,:) = - D.C.*sign(D.uk(2:D.Ny-2,:)).*(D.uk(2:D.Ny-2,:).*(D.uk(3:D.Ny-1,:) - D.uk(1:D.Ny-3,:))*2/(2*D.dy).*D.Yh(2:D.Ny-2,:)./(D.y0 + D.Yh(2:D.Ny-2,:).^2)  ...
            - D.uk(2:D.Ny-2,:).^2./(D.y0 + D.Yh(2:D.Ny-2,:).^2));
    end
end


% D.laphY(2:D.Ny-2,:) = (D.hk(1:D.Ny-3,:) + D.hk(3:D.Ny-1,:) - 2*D.hk(2:D.Ny-2,:))/D.dy^2 ;
% D.laphY(1,:) = ( 2*D.hk(2,:) - 2*D.hk(1,:))/D.dy^2 ;
% D.laphY(D.Ny-1,:) = ( 2*D.hk(D.Ny-2,:) - 2*D.hk(D.Ny-1,:))/D.dy^2 ;
% D.laphX(:,2:D.Nx-1) = (D.hk(:,1:D.Nx-2) + D.hk(:,3:D.Nx) - 2*D.hk(:,2:D.Nx-1))/D.dx^2 ;
% D.laphX(:,1)        = (D.hk(:,D.Nx) + D.hk(:,2) - 2*D.hk(:,1))/D.dx^2 ;
% D.laphX(:,D.Nx)     = (D.hk(:,D.Nx-1) + D.hk(:,1) - 2*D.hk(:,D.Nx))/D.dx^2 ;
% D.laph = D.laphX + D.laphY ;
% Divergence
D.divhY             = (D.vk(2:D.Ny,:) - D.vk(1:D.Ny-1,:))/D.dy ;
D.divhX(:,2:D.Nx-1) = (D.uk(:,3:D.Nx) - D.uk(:,1:D.Nx-2))/(2*D.dx) ;
D.divhX(:,1)        = (D.uk(:,2) - D.uk(:,D.Nx))/(2*D.dx) ;
D.divhX(:,D.Nx)     = (D.uk(:,1) - D.uk(:,D.Nx-1))/(2*D.dx) ;
D.divh = D.divhY + D.divhX ;
% set the integration time string HH:MM:SS
function str = runtime2str(t)
H = floor(t/3600)                                ; % Hours
M = floor((t-H*3600)/60)                         ; % Minutes
S = round(t-H*3600 - M*60)                       ; % Seconds
str = [int2str(H),':',int2str(M),':',int2str(S)] ; % Generate String
% Leap-Frog Time stepping
function D = LeapfrogFcn(D)
D.vk  = D.v ; D.hk  = D.h ; D.uk  = D.u ;
D = TimeStep1(D,2) ;
% Robert Asselin Time Filter
dv = D.as*(D.v+D.vlf - 2*D.vk) ;
du = D.as*(D.u+D.ulf - 2*D.uk) ;
dh = D.as*(D.h+D.hlf - 2*D.hk) ;
D.vlf = D.vk + dv*D.asModified ;
D.ulf = D.uk + du*D.asModified ;
D.hlf = D.hk + dh*D.asModified ;
D.v   = D.v + dv*(D.asModified-1) ;
D.u   = D.u + du*(D.asModified-1) ;
D.h   = D.h + dh*(D.asModified-1) ;
% Back Euler Time-Stepping
function D = Euler1Fcn(D)
D.vk  = D.v ; D.hk  = D.h ; D.uk  = D.u ;
D.vlf = D.v ; D.hlf = D.h ; D.ulf = D.u ;
D = TimeStep1(D,1) ;
% Time Step Liner Model, Wall B.C.
function D = TimeStep1(D,n)
D = CalculateVorDiv(D) ;
% time-step v
D.v(2:D.Ny-1,:) = D.vlf(2:D.Ny-1,:)   + n*D.dt * ( ...
    - D.Y(2:D.Ny-1,:).*(D.uk(1:D.Ny-2,:) + D.uk(2:D.Ny-1,:))/2 ...
    - (D.hk(2:D.Ny-1,:)-D.hk(1:D.Ny-2,:))/D.dy  - D.epsilon*D.v(2:D.Ny-1,:) ) ;
% time-step u
D.u(:,2:D.Nx-1) = D.ulf(:,2:D.Nx-1)   + n*D.dt * ( ...
    + D.Yh(:,2:D.Nx-1).*(D.vk(1:D.Ny-1,2:D.Nx-1) + D.vk(2:D.Ny,2:D.Nx-1))/2 ...
    - (D.hk(:,3:D.Nx)-D.hk(:,1:D.Nx-2))/(2*D.dx) - D.epsilon*D.uk(:,2:D.Nx-1) ) ;
% Periodic u BC
D.u(:,D.Nx) = D.ulf(:,D.Nx)   + n*D.dt * ( ...
    + D.Yh(:,D.Nx).*(D.vk(1:D.Ny-1,D.Nx) + D.vk(2:D.Ny,D.Nx))/2 ...
    - (D.hk(:,1)-D.hk(:,D.Nx-1))/(2*D.dx) - D.epsilon*D.uk(:,D.Nx)) ;
D.u(:,1) = D.ulf(:,1)   + n*D.dt * ( ...
    + D.Yh(:,1).*(D.vk(1:D.Ny-1,1) + D.vk(2:D.Ny,1))/2 ...
    - (D.hk(:,2)-D.hk(:,D.Nx))/(2*D.dx) - D.epsilon*D.uk(:,1)) ;
% time-step h
%D.h = D.hlf + n*D.dt * (- D.divh - (1-D.gamma)*(sign(-D.divh) +1)/2.*D.divh - D.epsilon*D.hk - D.Qx - D.Qy - D.Qo) ; % Convective coupling only for upward winds
D.h = D.hlf + n*D.dt * (- 1*D.divh + D.gamma.*((1+sign(D.divh))/2 - D.gammaFlag*(1 - sign(D.divh))/2).*D.divh +  - D.epsilon*D.hk - D.Qx - D.Qy - D.Qo) ;

if D.ZonalDiffusion > 0
    D.h(:,2:D.Nx-1) = D.h(:,2:D.Nx-1)   + n*D.dt*D.ZonalDiffusion/D.dx^2 * ( ...
        D.hk(:,3:D.Nx) + D.hk(:,1:D.Nx-2) - 2*D.hk(:,2:D.Nx-1)) ;
    D.h(:,1)    = D.h(:,1)   + n*D.dt*D.ZonalDiffusion/D.dx^2 * ( ...
        D.hk(:,2) + D.hk(:,D.Nx) - 2*D.hk(:,1)) ;
    D.h(:,D.Nx) = D.h(:,D.Nx)   + n*D.dt*D.ZonalDiffusion/D.dx^2 * ( ...
        D.hk(:,1) + D.hk(:,D.Nx-1) - 2*D.hk(:,D.Nx)) ;
    D.v(:,2:D.Nx-1) = D.v(:,2:D.Nx-1)   + n*D.dt*D.ZonalDiffusion/D.dx^2 * ( ...
        D.vk(:,3:D.Nx) + D.vk(:,1:D.Nx-2) - 2*D.vk(:,2:D.Nx-1)) ;
    D.v(:,1)        = D.v(:,1)   + n*D.dt*D.ZonalDiffusion/D.dx^2 * ( ...
        D.vk(:,2) + D.vk(:,D.Nx) - 2*D.vk(:,1)) ;
    D.v(:,D.Nx)     = D.v(:,D.Nx)   + n*D.dt*D.ZonalDiffusion/D.dx^2 * ( ...
        D.vk(:,1) + D.vk(:,D.Nx-1) - 2*D.vk(:,D.Nx)) ;
    D.u(:,2:D.Nx-1) = D.u(:,2:D.Nx-1)   + n*D.dt*D.ZonalDiffusion/D.dx^2 * ( ...
        D.uk(:,3:D.Nx) + D.uk(:,1:D.Nx-2) - 2*D.uk(:,2:D.Nx-1)) ;
    D.u(:,1)        = D.u(:,1)   + n*D.dt*D.ZonalDiffusion/D.dx^2 * ( ...
        D.uk(:,2) + D.uk(:,D.Nx) - 2*D.uk(:,1)) ;
    D.u(:,D.Nx)     = D.u(:,D.Nx)   + n*D.dt*D.ZonalDiffusion/D.dx^2 * ( ...
        D.uk(:,1) + D.uk(:,D.Nx-1) - 2*D.uk(:,D.Nx)) ;
end
% set up User Interface
function D = CreateGUI(D)
% Panel of Buttons
controlshp = uipanel('FontSize',12,'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',[.0 .0 .3], ...
    'Position',[D.b_left - D.panelext,D.Hgui-D.Hedge-D.NumEdit*D.Hparts - D.panelext,D.TextLength+2*D.panelext,D.Hgui-2*D.Hedge+D.panelext - (D.NumLines-D.NumEdit)*D.Hparts]);
edithp     = uipanel('FontSize',12,'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',[.0 .0 .3], ...
    'Position',[(D.b_left - D.panelext + D.TextLength+2*D.panelext),D.Hgui-D.Hedge-D.NumEdit*D.Hparts - D.panelext,(D.EditLength+2*D.panelext + D.TextSpacing),D.Hgui-2*D.Hedge+D.panelext - (D.NumLines-D.NumEdit)*D.Hparts]);
%Time
Line = 1 ;
h_RunTimeText = uicontrol('Style', 'text','Units','normalized','pos',[D.b_left,D.Hgui-D.Hedge-Line*D.Hparts,D.TextLength,D.HbuttonHeight], 'string','RunTime');
uicontrol(h_RunTimeText) ;  set(h_RunTimeText,'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',[.1 .1 .1],'FontSize',12,'FontAngle','oblique','Fontweight','bold', ...
    'TooltipString','Integration Run-time in HH:MM:SS') ;
D.h_RunTime = uicontrol('style','text','Units','normalized','pos',[D.b_left+D.TextSpacing+D.TextLength,D.Hgui-D.Hedge-Line*D.Hparts,D.EditLength,D.HbuttonHeight], 'string',num2str(D.ts), ...
    'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',13); uicontrol(D.h_RunTime) ;
Line = 2 ;
h_TimeText = uicontrol('Style', 'text','Units','normalized','pos',[D.b_left,D.Hgui-D.Hedge-Line*D.Hparts,D.TextLength,D.HbuttonHeight], 'string','Time(days)', 'BackgroundColor',D.FigBackgroudColor, ...
    'ForegroundColor',[.1 .1 .1],'FontSize',12,'FontAngle','oblique','Fontweight','bold', 'TooltipString','Integration Progress in Days'); uicontrol(h_TimeText) ;
D.h_Time = uicontrol('style','text','Units','normalized','pos',[D.b_left+D.TextSpacing+D.TextLength,D.Hgui-D.Hedge-Line*D.Hparts,D.EditLength,D.HbuttonHeight], 'string',num2str(D.t), ...
    'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',13) ;uicontrol(D.h_Time) ;
Line = 3 ;
h_EndTimeText = uicontrol('Style', 'text','Units','normalized','pos',[D.b_left,D.Hgui-D.Hedge-Line*D.Hparts,D.TextLength,D.HbuttonHeight], 'string','End(days)','BackgroundColor',D.FigBackgroudColor,...
    'ForegroundColor',[.1 .1 .1],'FontSize',12,'FontAngle','oblique','Fontweight','bold', 'TooltipString','Set here the duration of the integration in days') ; uicontrol(h_EndTimeText) ;
D.h_EndTime = uicontrol('style','edit','Units','normalized','pos',[D.b_left+D.TextSpacing+D.TextLength,D.Hgui-D.Hedge-Line*D.Hparts,D.EditLength,D.HbuttonHeight], 'string',num2str(D.EndTime),  'callback','set(gcbf,''userdata'',21); uiresume;');
uicontrol(D.h_EndTime) ;  set(D.h_EndTime,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',13) ;

Line = 4 ;
h_NxText = uicontrol('Style', 'text','Units','normalized','pos',[D.b_left,D.Hgui-D.Hedge-Line*D.Hparts,D.TextLength,D.HbuttonHeight], 'string','Nx','BackgroundColor',D.FigBackgroudColor, ...
    'ForegroundColor',[.1 .1 .1],'FontSize',12,'FontAngle','oblique','Fontweight','bold','TooltipString','Longitude Spatial resolution') ; uicontrol(h_NxText) ;
D.h_Nx = uicontrol('style','edit','Units','normalized','pos',[D.b_left+D.TextSpacing+D.TextLength,D.Hgui-D.Hedge-Line*D.Hparts,D.EditLength,D.HbuttonHeight], 'string',int2str(D.Nx),  ...
    'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',13) ; uicontrol(D.h_Nx) ;
Line = 5 ;
h_NyText = uicontrol('Style', 'text','Units','normalized','pos',[D.b_left,D.Hgui-D.Hedge-Line*D.Hparts,D.TextLength,D.HbuttonHeight], 'string','Ny', ...
    'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',[.1 .1 .1],'FontSize',12,'FontAngle','oblique','Fontweight','bold', ...
    'TooltipString','Latitude Spatial resolution') ;uicontrol(h_NyText) ;
D.h_Ny = uicontrol('style','edit','Units','normalized','pos',[D.b_left+D.TextSpacing+D.TextLength,D.Hgui-D.Hedge-Line*D.Hparts,D.EditLength,D.HbuttonHeight], 'string',int2str(D.Ny),  ...
    'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',13) ; uicontrol(D.h_Ny) ;
Line = 6 ;
h_dtText = uicontrol('Style', 'text','Units','normalized','pos',[D.b_left,D.Hgui-D.Hedge-Line*D.Hparts,D.TextLength,D.HbuttonHeight], 'string','dt (sec)', ...
    'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',[.1 .1 .1],'FontSize',12,'FontAngle','oblique','Fontweight','bold', ...
    'TooltipString','Integration Time-step (seconds)') ;  uicontrol(h_dtText) ;
D.h_dt = uicontrol('style','text','Units','normalized','pos',[D.b_left+D.TextSpacing+D.TextLength,D.Hgui-D.Hedge-Line*D.Hparts,D.EditLength,D.HbuttonHeight], 'string',num2str(D.dt,6), ...
    'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12) ;  uicontrol(D.h_dt) ;
Line = 7 ;
h_EpsilonText = uicontrol('Style', 'text','Units','normalized','pos',[D.b_left,D.Hgui-D.Hedge-Line*D.Hparts,D.TextLength,D.HbuttonHeight], 'string','\epsilon', ...
    'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',[.1 .1 .1],'FontSize',12,'FontAngle','oblique','Fontweight','bold', ...
    'TooltipString','Viscosity') ;  uicontrol(h_EpsilonText) ;
D.h_epsilon = uicontrol('style','text','Units','normalized','pos',[D.b_left+D.TextSpacing+D.TextLength,D.Hgui-D.Hedge-Line*D.Hparts,D.EditLength,D.HbuttonHeight], 'string',num2str(D.epsilon,6), ...
    'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12) ;  uicontrol(D.h_epsilon) ;
Line = 8 ;
h_GammaText = uicontrol('Style', 'text','Units','normalized','pos',[D.b_left,D.Hgui-D.Hedge-Line*D.Hparts,D.TextLength,D.HbuttonHeight], 'string','\gamma', ...
    'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',[.1 .1 .1],'FontSize',12,'FontAngle','oblique','Fontweight','bold', ...
    'TooltipString','Convective coupling') ;  uicontrol(h_GammaText) ;
D.h_gamma = uicontrol('style','text','Units','normalized','pos',[D.b_left+D.TextSpacing+D.TextLength,D.Hgui-D.Hedge-Line*D.Hparts,D.EditLength,D.HbuttonHeight], 'string',num2str(D.gamma,6), ...
    'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12) ;  uicontrol(D.h_gamma) ;
Line = 9 ;
h_SigYText = uicontrol('Style', 'text','Units','normalized','pos',[D.b_left,D.Hgui-D.Hedge-Line*D.Hparts,D.TextLength,D.HbuttonHeight], 'string','\sigma_y', ...
    'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',[.1 .1 .1],'FontSize',12,'FontAngle','oblique','Fontweight','bold', ...
    'TooltipString','Meridional Gaussian width') ;  uicontrol(h_SigYText) ;
D.h_sigy = uicontrol('style','text','Units','normalized','pos',[D.b_left+D.TextSpacing+D.TextLength,D.Hgui-D.Hedge-Line*D.Hparts,D.EditLength,D.HbuttonHeight], 'string',num2str(D.SigY,6), ...
    'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12) ;  uicontrol(D.h_sigy) ;
Line = 10 ;
h_AxText = uicontrol('Style', 'text','Units','normalized','pos',[D.b_left,D.Hgui-D.Hedge-Line*D.Hparts,D.TextLength,D.HbuttonHeight], 'string','A_x', ...
    'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',[.1 .1 .1],'FontSize',12,'FontAngle','oblique','Fontweight','bold', ...
    'TooltipString','Zonal Cos amplitude') ;  uicontrol(h_AxText) ;
D.h_Ax = uicontrol('style','text','Units','normalized','pos',[D.b_left+D.TextSpacing+D.TextLength,D.Hgui-D.Hedge-Line*D.Hparts,D.EditLength,D.HbuttonHeight], 'string',num2str(D.Ax,6), ...
    'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12) ;  uicontrol(D.h_Ax) ;
Line = 11 ;
h_Ay0Text = uicontrol('Style', 'text','Units','normalized','pos',[D.b_left,D.Hgui-D.Hedge-Line*D.Hparts,D.TextLength,D.HbuttonHeight], 'string','A_{y0}', ...
    'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',[.1 .1 .1],'FontSize',12,'FontAngle','oblique','Fontweight','bold', ...
    'TooltipString','Hemispherically symmetric Gaussian amplitude') ;  uicontrol(h_Ay0Text) ;
D.h_Ay0 = uicontrol('style','text','Units','normalized','pos',[D.b_left+D.TextSpacing+D.TextLength,D.Hgui-D.Hedge-Line*D.Hparts,D.EditLength,D.HbuttonHeight], 'string',num2str(D.Ay0,6), ...
    'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12) ;  uicontrol(D.h_Ay0) ;
Line = 12 ;
h_Ay1Text = uicontrol('Style', 'text','Units','normalized','pos',[D.b_left,D.Hgui-D.Hedge-Line*D.Hparts,D.TextLength,D.HbuttonHeight], 'string','A_{y1}', ...
    'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',[.1 .1 .1],'FontSize',12,'FontAngle','oblique','Fontweight','bold', ...
    'TooltipString','Zonal Cos amplitude') ;  uicontrol(h_Ay1Text) ;
D.h_Ay1 = uicontrol('style','text','Units','normalized','pos',[D.b_left+D.TextSpacing+D.TextLength,D.Hgui-D.Hedge-Line*D.Hparts,D.EditLength,D.HbuttonHeight], 'string',num2str(D.Ay1,6), ...
    'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12) ;  uicontrol(D.h_Ay1) ;
Line = 13 ;
h_CText = uicontrol('Style', 'text','Units','normalized','pos',[D.b_left,D.Hgui-D.Hedge-Line*D.Hparts,D.TextLength,D.HbuttonHeight], 'string','C', ...
    'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',[.1 .1 .1],'FontSize',12,'FontAngle','oblique','Fontweight','bold', ...
    'TooltipString','Ekman coupling coefficient') ;  uicontrol(h_CText) ;
D.h_C = uicontrol('style','text','Units','normalized','pos',[D.b_left+D.TextSpacing+D.TextLength,D.Hgui-D.Hedge-Line*D.Hparts,D.EditLength,D.HbuttonHeight], 'string',num2str(D.C,6), ...
    'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12) ;  uicontrol(D.h_C) ;
Line = 14 ;
h_C0Text = uicontrol('Style', 'text','Units','normalized','pos',[D.b_left,D.Hgui-D.Hedge-Line*D.Hparts,D.TextLength,D.HbuttonHeight], 'string','y0', ...
    'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',[.1 .1 .1],'FontSize',12,'FontAngle','oblique','Fontweight','bold', ...
    'TooltipString','Width of equatorial belt of weak Ekman coupling') ;  uicontrol(h_C0Text) ;
D.h_C0 = uicontrol('style','text','Units','normalized','pos',[D.b_left+D.TextSpacing+D.TextLength,D.Hgui-D.Hedge-Line*D.Hparts,D.EditLength,D.HbuttonHeight], 'string',num2str(D.y0,6), ...
    'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12) ;  uicontrol(D.h_C0) ;
for i = 1:D.NumEdit-1
    Line = i ;
    uipanel('FontSize',12,'BackgroundColor',[0 0 0],'ForegroundColor',[0 0 0], ...
        'Position',[D.b_left - D.panelext,D.Hgui-D.Hedge-Line*D.Hparts - D.linext ,(D.TextLength + D.TextSpacing + D.EditLength +4*D.panelext),D.panelext]);
end
% Defaults
function D = Defaults(D)
% Values:
%1. Comparable results to Gill are obtained for epsilon = 0.05 with gill forcing
%2. Comparable results to Matsuno are obtained for epsilon = 0.05 with Matsuno Forcing
%3. A double-ITCZ appears for epsilon >=0.2 for Matsuno forcing
D.epsilon   = .5                             ; % ND friction of U
D.C         =  0                             ; % Constant for coupling to laplacian
D.Cx        = 0                              ;
D.gamma     = 1                              ; % coupling of convection to vertical wind. 0--1. 1 = no coupling.
D.Omega   = 2*pi/(24*3600)                   ; % rotational frequency of Earth
D.Radius  = 6.4e6                            ; % Earth's Radius
D.g       = 9.81                             ; % Gravitational Constant
D.Day     = 24*3600                          ; % Day in Seconds
D.as      = 0.001                            ; % Robert Asselin Time Filter
D.asModified = 1                             ; % =.5 Modified Rober-Asselin-Williams Filter. 1 = Standard Robert-Asselin Filter
D.ZonalDiffusion = 0                         ; % Zonal 2nd order diffusion coefficient
% Independent Consts
D.Lx   = 25                                  ; % longitude domain length
D.Ly   = 5                                   ; % Northern Boundary of Channel
D.EndTime = 100                              ; % End Intergation Time in normalized time units
D.t       = 0                                ; % Time (tau units)
D.ts      = 0                                ; % Timer-seconds
D.dt_adjust_ratio        = 1.3              ; % When instability occurs, dt is reduced by this ratio
D.stability_period_ratio = 1/3               ; % portion of total number of time steps at which stability check is performed
% Spatial
% D.Nx  = 5                                   ; % Number of longitude grid points
% D.Ny  = 100                                   ; % Number of meridional grid points
D.Nx  = 200                                   ; % Number of longitude grid points
D.Ny  = 80                                   ; % Number of meridional grid points D.dt      = 5.e-3                            ; % default time step
D.dt      = 5.e-3                            ; % default time step
% Display and Numerical
D.modPlotStep    = 200                       ; % frame refresh
D.frames_per_day = 4                         ; % number of frames per day of integration
D.PlaybackRefreshInterval = 0.1              ; % refresh interval between frames during Playback
D.DisplayIdx     = 1                         ; % Display Options
D.LineWidth      = 0                         ;
D.Xcrossection   = .5                        ; % Crossection point along x ;
D.u_ContourLevelStep = 0                     ; % Contour levelstep. 0 = Auto
D.h_ContourLevelStep = 0                     ; % Contour levelstep. 0 = Auto
D.v_ContourLevelStep = 0                     ; % Contour levelstep. 0 = Auto
D.ColorMaps.BlueWhiteRed = cejulienColorMaps('BlueWhiteRed',23) ;
D.ColorMaps.RYBceJulien  = cejulienColorMaps('RYBceJulien',27) ;
D.ColorMaps.OWBceJulien  = cejulienColorMaps('OWBceJulien',27) ;
D.ColorMaps.TWBceJulien  = cejulienColorMaps('TWBceJulien',27) ;
D.ColorMaps.OTBceJulien  = cejulienColorMaps('OTBceJulien',27) ;
% GUI Parameters
D.SaveFileFlag           = 1 ; % Flag for saving run data to file
D.SavePlotFlag           = 0 ; % Flag for creating and saving a plot of the final frame
D.scrsz      = get(0,'ScreenSize')                       ;
D.pos        = [D.scrsz(3)*.05 D.scrsz(4)*.05 D.scrsz(3)*.75 D.scrsz(4)*.75] ;
D.FigBackgroudColor = [.91 .92 .92]   ; % Figure Backgroup colow
D.NumLines      = 21                  ; % Number of panel lines
D.NumEdit       = 14                  ; % Number of framed panel lines
D.Hgui          = 1                   ; % Height dimension of GUI
D.Hedge         = 2/100               ; % Top and bottom edges distnces of GUI
D.Hparts = (D.Hgui-2*D.Hedge )/D.NumLines ; % Height of each panel line
D.HbuttonHeight = D.Hparts*0.8        ; % height of each button in normalized units
D.HbuttonLength = 5/100               ; % Length of buttons in normalized units
D.TextLength    = 7/100               ; % text length in normalized units
D.EditLength    = 6/100               ; % Edit box length in normalized units
D.TextSpacing   = .5/100              ; % Text spacing in normalized units
D.b_left        = .5/100              ; % left indent in normalized units
D.panelext      = .2/100              ; % panel edges in normalized units
D.linext        = .1*D.Hparts         ; % Line edges in ormalized units
% Set Dependent Consts
D = Dependent_Consts(D)                       ; % Set Dependent Consts
% Update dependent consts
function D = Dependent_Consts(D)
% Spatial
D.x    = linspace(0,D.Lx,D.Nx+1)          ;
D.x    = D.x(1:end-1)                     ;
D.dx   = D.x(2)-D.x(1)                    ; % longitudinal spatial resolution
D.y    = linspace(-D.Ly,D.Ly,D.Ny)         ;
D.dy   = D.y(2)-D.y(1)                    ; % longitudinal spatial resolution
D.yh   = (D.y(1:end-1) + D.y(2:end))/2    ; % Staggared Grid for h, u
[D.X ,D.Y ] = meshgrid(D.x,D.y)           ;
[D.Xh,D.Yh] = meshgrid(D.x,D.yh)          ;
D.Ex = D.Xh/D.Lx*D.Cx                      ;
D.Tend = D.EndTime                        ; % End integration time in time units
D.N    = ceil(D.Tend/D.dt)                ; % number of time steps
D.dt0  = D.dt                             ; % recording default time resolutions
D.dt0_Abs = D.dt                          ;
% Settings
function D = RunSimulation(D)
Auto_dt_adjust_Flag    = 1               ; % Keeps running so lon flag = 1
while Auto_dt_adjust_Flag
    tic ;
    N_current = D.N                ; % Record current total number of steps
    D = InitialConditions(D)       ; % set initial conditions
    D = Dependent_Consts(D)        ;
    % Initialize using Forward Euler
    D.t = 0                                          ; % Time
    D.ts = 0                                         ; % Timer-seconds
    D.t = D.t + D.dt                                 ;
    f = 1 ; % Integration iteration index
    % Initial Step Using Forward Euler
    D = Euler1Fcn(D); % Initial step to initialize the integration before continuing with Leap-frog
    % Display Initial Step
    frame_plot(D) ;
    set(D.h_Time,'string',num2str(D.t/D.Day,3)) ;
    % Integrate using Leap-Frog
    N_Stability_Check = round(D.N*D.stability_period_ratio) ; %
    % Main Loop
    D.PauseFlag = 0 ;
    for i= 1: D.N-1
        D.t = D.t + D.dt     ;
        D = LeapfrogFcn(D) ; % Leap-Frog Timestep
        if D.t > f*D.nframes*D.dt || mod(i,D.modPlotStep) == 1
            D = Euler1Fcn(D); % Occational Euler stepping to destroy destabilizing transient modes
            try
                frame_plot(D) ;
            end
            set(D.h_Time,'string',num2str(D.t,3)) ;
            D.ts = toc ;
            runtime = runtime2str(toc) ;
            set(D.h_RunTime,'string',runtime) ;
            if get(gcf,'userdata') == 1  ;
                Auto_dt_adjust_Flag = 0  ;
                D.PauseFlag = 1  ;
                break ;
            end
            drawnow ;
            if D.t > f*D.nframes*D.dt
                f=f+1 ;
            end
        end
        if mod(i,5)==1 && i < N_Stability_Check
            if sum(nanmean(abs(D.v),1)) > 100
                disp (['Integration Restarted after ',int2str(i),' steps due to instability. dt--> dt/',num2str(D.dt_adjust_ratio,3)]) ;
                D.dt = D.dt/D.dt_adjust_ratio ;
                set(D.h_dt,'String',num2str(D.dt,5))  ; D.N = round(D.Tend/D.dt)  ; drawnow ;
                break ;
            end
        end
        
    end
    if i > N_current - 2 ; Auto_dt_adjust_Flag = 0 ; end
end
frame_plot(D) ; drawnow                                 ; % Update Screen
set(D.h_Time,'string',num2str(D.t/D.Day,3))             ; % Update runtime in years
D.ts = toc                                              ; % Save final time of run
runtime = runtime2str(toc)                              ; % convert tuntime to HH:MM:SS
set(D.h_RunTime,'string',runtime) ; drawnow             ; % Update run time
D.finalruntime = toc ;
% Save Data as structure S
D = CalculateVorDiv(D)  ;
S.t = D.t ;
S.u = D.u ;
S.v = D.v ;
S.h = D.h ;
S.vor = D.vor ;
S.vorh = D.vorh ;
S.divh = D.divh ;
S.ForcingString = D.ForcingString ;
S.Ekman_Flag    = D.Ekman_Flag    ;
S.Nx = D.Nx  ;
S.Ny = D.Ny  ;
S.Lx = D.Lx  ;
S.Ly = D.Ly  ;
S.x  = D.x   ;
S.dx = D.dx  ;
S.y  = D.y   ;
S.dy = D.dy  ;
S.yh = D.yh  ;
S.Xh = D.Xh  ;
S.Yh = D.Yh  ;
S.X  = D.X   ;
S.Y  = D.Y   ;
S.Qx = D.Qx  ;
S.Qy = D.Qy  ;
S.Qo = D.Qo  ;
S.epsilon  = D.epsilon  ;
S.gamma    = D.gamma    ;
S.Ax       = D.Ax       ;
S.Ay0      = D.Ay0      ;
S.Ay1      = D.Ay1      ;
S.C        = D.C        ;
S.y0       = D.y0       ;
S.SigY     = D.SigY     ;
S.ZonalDiffusion = D.ZonalDiffusion ;

savestr = ['Nx_',int2str(D.Nx),'_Ny_',int2str(D.Ny),'_',D.ForcingString]  ;
if D.SaveFileFlag && ~strcmp(D.ForcingString,'Test')
    try
        save(fullfile('Saved Runs',[savestr,'.mat']),'S') ;
    catch
        mkdir('Saved Runs') ;
        save(fullfile('Saved Runs',savestr,['.mat']),'S') ;
    end
    disp(savestr) ;
end
if D.SavePlotFlag
    close(D.MainFig) ;
    Fig = figure('position',D.pos.*[1 1 .7 .7],'color',[1 1 1]) ;
    frame_plot(D) ; drawnow                                 ; % Update Screen
    export_fig(savestr,'-pdf','-transparent','-opengl')
end
% Set initial conditions
function D = InitialConditions(D)
% Defualts
D.gamma = 0            ; % gamma = 1 = no convective coupling. gamma --> 0 ==> strong convective coupling
D.gammaFlag = 1        ; % when 0, convective coupling occurs only for upward winds
D.ZonalDiffusion = 0   ; % numerical zonal second order diffusion
D.Ekman_Flag = 1       ; % when 1 C=0 for westerly winds
D.Linear_Ekman_Flag = 1; % when 1 Ekamn coupling assumes the stress tau_x~u instead of the bulk formulation tau_x~|u|u
D.Total_Ekman_Flag = 1 ; % when 1 the total divergence of the ocean Ekman balance vo_y + uo_x is used
D.C      = 0           ; % Meridional Ekman coupling coefficient
D.Cx     = 0           ; % Zonal Ekman coupling coefficient
D.Cd     = 0           ; % Zonal conponent of surface drag driving zonal ocean overturning circulation
D.y0     = 0.2        ; % latitudial belt where Ekman balance does not hold
D.SigY   = 1           ; % Sharpness of forcing distribution at the equator
D.Ax     = 0           ; % zonal pressuren gradient amplitude
D.Ay0    = 0           ; % Amplitude of equatorual heating anomaly for hemispherically symmetric forcing
D.Ay1    = 0           ; % Amplitude of interhemispheric pressure gradient for hemispherically antisymmetric forcing
% CASES
%D.ForcingString = 'Test' ;
%D.ForcingString = 'Gill_Symmetric' ;
%D.ForcingString = 'Hadley_Sol_Damped' ;
%D.ForcingString = 'Walker_Damped3_Convective2' ; 
%D.ForcingString = 'Symmetric_JAS' ;
if ~isfield(D,'ForcingString')
    D.ForcingString = 'Test' ;
end
switch D.ForcingString(1:3)
    case {'Wal','Had','Sol'}
        switch D.ForcingString
            case 'Walker_Matsuno'
                D.Ax = -0.1 ;
                D.epsilon = 0.01 ;
            case 'Walker_Damped1'
                D.Ax = -0.1 ;
                D.epsilon = 0.1 ;
            case 'Walker_Damped2'
                D.Ax = -0.1 ;
                D.epsilon = 0.5 ;
            case 'Walker_Damped3'
                D.Ax = -0.1 ;
                D.epsilon = 1 ;
            case 'Walker_Convective'
                D.Ax = -0.1 ;
                D.epsilon = 0.01 ;
                D.gamma = .5 ;
            case 'Walker_Damped1_Convective'
                D.Ax = -0.1 ;
                D.epsilon = 0.1 ;
                D.gamma = .1 ;
            case 'Walker_Damped2_Convective'
                D.Ax = -0.1 ;
                D.epsilon = 0.5 ;
                D.gamma = .5 ;
            case 'Walker_Damped3_Convective'
                D.Ax = -0.1 ;
                D.epsilon = 1 ;
                D.gamma = .5 ;
            case 'Walker_Damped3_Convective2'
                D.Ax = -0.1 ;
                D.epsilon = 1 ;
                D.gamma = .9 ;    
            case 'Walker_Damped1_Ekman1'
                D.Ax = -0.1 ;
                D.epsilon = 0.1 ;
                D.C = .05 ;
            case 'Walker_Damped1_Ekman2'
                D.Ax = -0.1 ;
                D.epsilon = 0.1 ;
                D.C = .1 ;
            case 'Walker_Damped1_Ekman3'
                D.Ax = -0.1 ;
                D.epsilon = 0.1 ;
                D.C = 0.15 ;
            case 'Walker_Damped1_Ekman4'
                D.Ax = -0.1 ;
                D.epsilon = 0.1 ;
                D.C = .2 ;
            case 'Walker_Damped1_Ekman5'
                D.Ax = -0.1 ;
                D.epsilon = 0.1 ;
                D.C = .25 ;
            case 'Walker_Damped2_Ekman1'
                D.Ax = -0.1 ;
                D.epsilon = 0.5 ;
                D.C = .05 ;
            case 'Walker_Damped2_Ekman2'
                D.Ax = -0.1 ;
                D.epsilon = 0.5 ;
                D.C = .1 ;
            case 'Walker_Damped2_Ekman3'
                D.Ax = -0.1 ;
                D.epsilon = 0.5 ;
                D.C = .15 ;
            case 'Walker_Damped2_Ekman4'
                D.Ax = -0.1 ;
                D.epsilon = 0.5 ;
                D.C = .2 ;
            case 'Walker_Damped2_Ekman5'
                D.Ax = -0.1 ;
                D.epsilon = 0.5 ;
                D.C = .25 ;
            case 'Walker_Damped3_Ekman1'
                D.Ax = -0.1 ;
                D.epsilon = 1 ;
                D.C = .05 ;
            case 'Walker_Damped3_Ekman2'
                D.Ax = -0.1 ;
                D.epsilon = 1 ;
                D.C = .1 ;
            case 'Walker_Damped3_Ekman3'
                D.Ax = -0.1 ;
                D.epsilon = 1 ;
                D.C = .15 ;
            case 'Walker_Damped3_Ekman4'
                D.Ax = -0.1 ;
                D.epsilon = 1 ;
                D.C = .2 ;
            case 'Walker_Damped3_Ekman5'
                D.Ax = -0.1 ;
                D.epsilon = 1 ;
                D.C = .25 ;
            case 'Walker_Damped3_Ekman1_Convective'
                D.Ax = -0.1 ;
                D.epsilon = 1 ;
                D.C = .05 ;
                D.gamma = .5 ;
            case 'Walker_Damped3_Ekman2_Convective'
                D.Ax = -0.1 ;
                D.epsilon = 1 ;
                D.C = .1 ;
                D.gamma = .5 ;
            case 'Walker_Damped3_Ekman3_Convective'
                D.Ax = -0.1 ;
                D.epsilon = 1 ;
                D.C = .15 ;
                D.gamma = .5 ;
            case 'Walker_Damped3_Ekman4_Convective'
                D.Ax = -0.1 ;
                D.epsilon = 1 ;
                D.C = .2 ;
                D.gamma = .5 ;
            case 'Walker_Damped3_Ekman5_Convective'
                D.Ax = -0.1 ;
                D.epsilon = 1 ;
                D.C = .25 ;
                D.gamma = .5 ;
            case 'Walker_Sweep1_1'
                D.Ax = -0.1 ;
                D.SigY = 1 ;
                D.epsilon = 1 ;
                D.C = .2 ;
            case 'Walker_Sweep1_2'
                D.Ax = -0.1 ;
                D.SigY = 1.5 ;
                D.epsilon = 1 ;
                D.C = .2 ;
            case 'Walker_Sweep1_3'
                D.Ax = -0.1 ;
                D.SigY = 2 ;
                D.epsilon = 1 ;
                D.C = .2 ;
            case 'Walker_Sweep1_4'
                D.Ax = -0.2 ;
                D.SigY = 1 ;
                D.epsilon = 1 ;
                D.C = .2 ;
            case 'Walker_Sweep1_5'
                D.Ax = -0.2 ;
                D.SigY = 1.5 ;
                D.epsilon = 1 ;
                D.C = .2 ;
            case 'Walker_Sweep1_6'
                D.Ax = -0.2 ;
                D.SigY = 2 ;
                D.epsilon = 1 ;
                D.C = .2 ;
            case 'Walker_Sweep1_7'
                D.Ax = -0.3 ;
                D.SigY = 1 ;
                D.epsilon = 1 ;
                D.C = .2 ;
            case 'Walker_Sweep1_8'
                D.Ax = -0.3 ;
                D.SigY = 1.5 ;
                D.epsilon = 1 ;
                D.C = .2 ;
            case 'Walker_Sweep1_9'
                D.Ax = -0.3 ;
                D.SigY = 2 ;
                D.epsilon = 1 ;
                D.C = .2 ;    
            case 'HadleyWalker_Damped1'
                D.Ax = -0.1 ;
                D.Ay0 = -0.05 ;
                D.epsilon = .1 ;
            case 'HadleyWalker_Damped3'
                D.Ax = -0.1 ;
                D.Ay0 = -0.05 ;
                D.epsilon = 1 ;
            case 'HadleyWalker_Damped3_Ekman1_Convective2'
                D.Ax = -0.2 ;
                D.Ay0 = -0.1 ;
                D.epsilon = 1 ;
                D.C = 1 ;
                D.gamma = .1 ;
            case 'HadleyWalker_Damped3_Ekman2_Convective2'
                D.Ax = -0.2 ;
                D.Ay0 = -0.1 ;
                D.epsilon = 1 ;
                D.C = 2 ;
                D.gamma = .1 ;
            case 'HadleyWalker_Damped3_Ekman3_Convective2'
                D.Ax = -0.2 ;
                D.Ay0 = -0.1 ;
                D.epsilon = 1 ;
                D.C = 3 ;
                D.gamma = .1 ;
            case 'HadleyWalker_Damped3_Ekman4_Convective2'
                D.Ax = -0.2 ;
                D.Ay0 = -0.1 ;
                D.epsilon = 1 ;
                D.C = 4 ;
                D.gamma = .1 ;
            case 'HadleyWalker_Damped3_Ekman5_Convective2'
                D.Ax = -0.2 ;
                D.Ay0 = -0.1 ;
                D.epsilon = 1 ;
                D.C = 5 ;
                D.gamma = .1 ;
            case 'HadleyWalker_Damped3_Ekman6_Convective2'
                D.Ax = -0.2 ;
                D.Ay0 = -0.1 ;
                D.epsilon = 1 ;
                D.C = 6 ;
                D.gamma = .1 ;
            case 'HadleyWalker_Damped3_EkmanE1_Convective2'
                D.Ax = -0.2 ;
                D.Ay0 = -0.1 ;
                D.epsilon = 1 ;
                D.C = 1 ;
                D.gamma = .1 ;
                D.Ekman_Flag = 1       ;
            case 'HadleyWalker_Damped3_EkmanE2_Convective2'
                D.Ax = -0.2 ;
                D.Ay0 = -0.1 ;
                D.epsilon = 1 ;
                D.C = 2 ;
                D.gamma = .1 ;
                D.Ekman_Flag = 1       ;
            case 'HadleyWalker_Damped3_EkmanE3_Convective2'
                D.Ax = -0.1 ;
                D.Ay0 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 3 ;
                D.gamma = .1 ;
                D.Ekman_Flag = 1       ;
            case 'HadleyWalker_Damped3_EkmanE4_Convective2'
                D.Ax = -0.2 ;
                D.Ay0 = -0.1 ;
                D.epsilon = 1 ;
                D.C = 4 ;
                D.gamma = .1 ;
                D.Ekman_Flag = 1       ;
            case 'HadleyWalker_Damped3_EkmanE3_Convective2_ZD'
                D.Ax = -0.1 ;
                D.Ay0 = 0.05 ;
                D.epsilon = 1 ;
                D.C = 3 ;
                D.gamma = .1 ;
                D.Ekman_Flag = 1       ;
                D.ZonalDiffusion = 0   ;
            case 'HadleyWalker_Damped3_EkmanE6_Convective2_ZD'
                D.Ax = -0.02 ;
                D.Ay0 = -0.01 ;
                D.epsilon = 1 ;
                D.C = 6 ;
                D.gamma = .1 ;
                D.Ekman_Flag = 1       ;
                D.ZonalDiffusion = 0.01   ;
            case 'SolWalker_Damped3_Ekman1_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 1 ;
                D.gamma = .1 ;
            case 'SolWalker_Damped3_Ekman2_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 2 ;
                D.gamma = .1 ;
            case 'SolWalker_Damped3_Ekman3_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 3 ;
                D.gamma = .1 ;
            case 'SolWalker_Damped3_Ekman4_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 4 ;
                D.gamma = .1 ;
            case 'SolWalker_Damped3_Ekman5_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 5 ;
                D.gamma = .1 ;
            case 'SolWalker_Damped3_Ekman6_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 6 ;
                D.gamma = .1 ;
            case 'SolWalker_Damped3_EkmanE1_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 1 ;
                D.gamma = .1 ;
                D.Ekman_Flag = 1       ;
            case 'SolWalker_Damped3_EkmanE2_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 2 ;
                D.gamma = .1 ;
                D.Ekman_Flag = 1       ;
            case 'SolWalker_Damped3_EkmanE3_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 3 ;
                D.gamma = .1 ;
                D.Ekman_Flag = 1       ;
            case 'SolWalker_Damped3_EkmanE4_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 4 ;
                D.gamma = .1 ;
                D.Ekman_Flag = 1       ;
            case 'SolWalker_Damped3_EkmanE5_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 5 ;
                D.gamma = .1 ;
                D.Ekman_Flag = 1       ;
            case 'SolWalker_Damped3_EkmanE6_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 6 ;
                D.gamma = .1 ;
                D.Ekman_Flag = 1       ;
            case 'SolWalker_Damped1_Convective1'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = .1 ;
                D.gamma = .5 ;
            case 'SolWalker_Damped2_Convective1'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = .5 ;
                D.gamma = .5 ;
            case 'SolWalker_Damped3_Convective1'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.gamma = .5 ;
            case 'SolWalker_Damped1_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = .1 ;
                D.gamma = .1 ;
            case 'SolWalker_Damped2_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = .5 ;
                D.gamma = .1 ;
            case 'SolWalker_Damped3_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.gamma = .1 ;
            case 'SolWalker_Damped1'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = .1 ;
            case 'SolWalker_Damped2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = .5 ;
            case 'SolWalker_Damped3'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
            case 'Sol2Walker_Damped3'
                D.Ax = -0.1 ;
                D.Ay1 = -0.1 ;
                D.epsilon = 1 ;
            case 'Sol2Walker_Damped3_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.1 ;
                D.epsilon = 1 ;
                D.gamma = .1 ;
            case 'Sol2Walker_Damped3_Convective2_EkmanE3'
                D.Ax = -0.1 ;
                D.Ay1 = -0.1 ;
                D.epsilon = 1 ;
                D.C = 3       ;
                D.gamma = .1 ;
            case 'SolWalker_Damped3_EkmanE3'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 3       ;
            case 'HadleySolWalker_Damped3'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.Ay0 = -0.05 ;
                D.epsilon = 1 ;
            case 'HadleySolWalker_Damped3_EkmanE3'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.Ay0 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 3       ;
            case 'HadleySolWalker_Damped3_EkmanE5'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.Ay0 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 5       ;
            case 'HadleySolWalker_Damped3_EkmanE3_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.Ay0 = -0.05 ;
                D.epsilon = 1 ;
                D.C = .3       ;
                D.gamma = .1  ;
            case 'HadleySolWalker_Damped3_EkmanE5_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.Ay0 = -0.05 ;
                D.epsilon = 1 ;
                D.C = .5       ;
                D.gamma = .1  ;
            case 'HadleySolWalker_Damped3_EkmanE10_Convective2'
                D.Ax = -0.1 ;
                D.Ay1 = -0.05 ;
                D.Ay0 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 1       ;
                D.gamma = .1  ;
            case 'Hadley_Eq'
                D.Ay0 = -0.1 ;
                D.epsilon = 0.001 ;
            case 'Hadley_Eq_Damped'
                D.Ay0 = -0.1 ;
                D.epsilon = 0.1 ;
            case 'Hadley_Eq_Convective'
                D.Ay0 = -0.1 ;
                D.epsilon = 0.005 ;
                D.gamma = .01 ;
            case 'Hadley_Eq_Damped_Convective'
                D.Ay0 = -0.1 ;
                D.epsilon = 0.1 ;
                D.gamma = .01 ;
            case 'Hadley_Eq_Damped_EkmanE'
                D.Ay0 = -0.1 ;
                D.epsilon = 0.1 ;
                D.C = 2 ;
                D.Ekman_Flag = 1 ;
            case 'Hadley_Eq_Damped_Convective_Ekman'
                D.Ay0 = -0.1 ;
                D.epsilon = 0.1 ;
                D.C = 5 ;
                D.gamma = 0.1 ;
            case 'Hadley_Sol'
                D.Ay1 = -0.1 ;
                D.epsilon = 0.001 ;
            case 'Hadley_Sol_Damped'
                D.Ay1 = -0.1 ;
                D.epsilon = 0.1 ;
            case 'Hadley_Sol_Damped_Convective'
                D.Ay1 = -0.01 ;
                D.epsilon = 0.1 ;
                D.gamma = 0.1 ;
            case 'Hadley_Sol_Damped_Ekman'
                D.Ay1 = -0.01 ;
                D.epsilon = 0.1 ;
                D.C = 15 ;
            case 'Hadley_Sol_Damped_Convective_Ekman'
                D.Ay1 = -0.01 ;
                D.epsilon = 0.1 ;
                D.C = 5 ;
                D.gamma = 0.1 ;
            case 'HadleyWalker'
            case 'HadleyWalker_Damped'
            case 'HadleyWalker_Damped_Convective'
            case 'HadleyWalker_Damped_Ekman'
            case 'HadleyWalker_Damped_Convective_Ekman'
        end
        D.Qx   = -D.epsilon*D.Ax*sin(D.Xh*2*pi/D.Lx).*exp(-(D.Yh).^2/2/D.SigY^2) ;
        D.Qy   = -D.epsilon*(D.Ay0 + D.Ay1*sin(D.Yh*pi/D.Ly)).*exp(-(D.Yh).^2/2/D.SigY^2) ;
        if D.epsilon < 0.1 || D.C > .5
            D.EndTime = 200                              ; % End Intergation Time in normalized time units
        elseif D.epsilon <= 0.5
            D.EndTime = 100 ; 
        else
            D.EndTime = 30 ; % End Intergation Time in normalized time units
        end
        if D.ZonalDiffusion > .1
            D.EndTime = 20 ;
        end
        %D.EndTime = 100 ;
    case 'Gil'
        switch D.ForcingString
            case 'Gill_Symmetric'
                Ax = -0.01 ;
                SigX = 5 ;
                SigY = 2 ;
                D.epsilon = 0.01 ;
                D.alpha  = 0                                ; % 0.4 Moist vorticity damping coefficient
                D.epsilon0  = D.alpha*D.epsilon             ; % 0.4 Moist vorticity damping coefficient
                D.Qx  = -D.epsilon*Ax*exp(-(D.Xh -10).^2/SigX).*exp(-(D.Yh).^2/2/D.SigY^2)  ;
            case 'Gill_Symmetric_Zonal'
                Ax = -0.01 ;
                SigX = 5 ;
                SigY = 2 ;
                D.epsilon = 0.05 ;
                D.Qx  = -D.epsilon*(Ax*exp(-(D.Xh -10).^2/SigX).*exp(-(D.Yh).^2/2/D.SigY^2)  ...
                    -  Ax*exp(-(D.Xh -20).^2/SigX).*exp(-(D.Yh).^2/2/D.SigY^2))  ;
            case 'Gill_Symmetric_Damping'
                Ax = -0.01 ;
                SigX = 5 ;
                SigY = 2 ;
                D.epsilon = 0.1 ;
                D.gamma = .1 ;
                D.Qx  = -D.epsilon*Ax*exp(-(D.Xh -10).^2/SigX).*exp(-(D.Yh).^2/2/D.SigY^2)  ;
            case 'Gill_Symmetric_Convective'
                Ax = -0.01 ;
                SigX = 5 ;
                SigY = 2 ;
                D.epsilon = 0.5 ;
                D.Qx  = -D.epsilon*Ax*exp(-(D.Xh -10).^2/SigX).*exp(-(D.Yh).^2/2/D.SigY^2)  ;
        end
        D.Qy = 0 ;
    case 'Sym'
        switch D.ForcingString
            case 'Symmetric_MAM'
                D.Ax = -0.1 ;
                D.Ay0 = -0.05 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 0 ;
                D.gamma = .2 ;
                D.Ekman_Flag = 1       ;
                D.ZonalDiffusion = 0   ;
                D.Qx   = -D.epsilon*D.Ax*sin(D.Xh*2*pi/D.Lx).*exp(-(D.Yh).^2/2/D.SigY^2) ;
                D.Qy   = -D.epsilon*(D.Ay0 + D.Ay1*sin(D.Yh*pi/D.Ly).^2).*exp(-(D.Yh).^2/2/D.SigY^2) ;
            case 'Symmetric_JAS'
                D.Ax = -0.1 ;
                D.Ay0 = -0.05 ;
                D.Ay1 = -0.05 ;
                D.epsilon = 1 ;
                D.C = 1 ;
                D.gamma = .2 ;
                D.Ekman_Flag = 1       ;
                D.ZonalDiffusion = 0   ;
                D.Qx   = -D.epsilon*D.Ax*sin(D.Xh*2*pi/D.Lx).*exp(-(D.Yh).^2/2/D.SigY^2) ;
                D.Qy   = -D.epsilon*(D.Ay0 + D.Ay1*sin(D.Yh*pi/D.Ly).^2).*exp(-(D.Yh).^2/2/D.SigY^2) ;
        end
        D.EndTime = 50 ;
    case 'Tes'
        D.Ax = -0.1 ;
        D.Ay0 = 0 ;
        D.Ay1 = 0  ;
        D.epsilon = 1 ;
        D.y0     = 0.2 ;
        D.SigY   = 1 ;
        %D.C = .5 ;
        %D.Cx = 0.5 ;
        D.C = 0.0 ;
        D.gamma = 0 ;
        D.Ekman_Flag = 1       ;
        D.Total_Ekman_Flag = 1 ;
        D.Linear_Ekman_Flag = 1 ;
        D.ZonalDiffusion = 0   ;
        D.Qx   = -D.epsilon*D.Ax*sin(D.Xh*2*pi/D.Lx).*exp(-(D.Yh).^2/2/D.SigY^2) ;
        D.Qy   = -D.epsilon*(D.Ay0 + D.Ay1*sin(D.Yh*pi/D.Ly)).*exp(-(D.Yh).^2/2/D.SigY^2) ;
        %D.Qx   = -D.epsilon*D.Ax*sin(D.Xh*2*pi/D.Lx).*exp(-(D.Yh).^2/2/D.SigY^2) ;
        %D.Qy   = -D.epsilon*(D.Ay0 + D.Ay1*sin(D.Yh*pi/D.Ly).^2).*exp(-(D.Yh).^2/2/D.SigY^2) ;
        D.EndTime = 30 ;
end

% Set Initial Values
D.h  = zeros(D.Ny-1,D.Nx) ; % h Initial Condition
D.u  = zeros(D.Ny-1,D.Nx) ; % u Initial Condition
%D.u = D.u + (0.1*sin(D.Yh*pi/D.Ly) + 0.1).*exp(-(D.Yh).^2/2/D.SigY^2)  ;
D.v  = zeros(D.Ny,D.Nx)   ; % v Initial condition
D.vor  = zeros(D.Ny,D.Nx)   ; % vorticity Initial condition
D.vorh = zeros(D.Ny-1,D.Nx)   ; % vorticity Initial condition
D.divh = zeros(D.Ny-1,D.Nx)   ; % Divergence Initial condition
D.divhX = zeros(D.Ny-1,D.Nx)   ; % Divergence Initial condition
D.divhY = zeros(D.Ny-1,D.Nx)   ; % Divergence Initial condition
D.vorQ = zeros(D.Ny-1,D.Nx)   ; % Convective Forcing initial condition
D.Qo = zeros(D.Ny-1,D.Nx)   ; % Convective Forcing initial condition
D.Qoz = zeros(D.Ny-1,D.Nx)   ; % Convective Forcing initial condition
D.vorU = zeros(D.Ny-1,D.Nx)   ; % Convective Forcing initial condition
D.vorV = zeros(D.Ny,D.Nx)   ; % Convective Forcing initial condition
D.Q    = zeros(D.Ny-1,D.Nx)   ; % Total Forcing initial condition
D = Dependent_Consts(D) ;
% Leap Frog Variables
D.uk = D.u ; D.ulf = D.u ;
D.hk = D.h ; D.hlf = D.h ;
D.vk = D.v ; D.vlf = D.v ;

% Setup for Saving Simulation
if D.Tend/D.Day >1
    D.frames = round(D.frames_per_day*D.Tend/D.Day); % number of times the result is Displayed and saved in real time
else
    D.frames = 24                             ; % default for durations shorter than a day (high temporal resolution)
end
D.nframes = round(D.N/(D.frames))             ; % number of timestep increments between frames
D.f     = 0                                   ; % frame index
